# frozen_string_literal: true

module Kubelink
  class Sentrylink < Rails::Railtie
    def self.sentry_dsn
      @dsn ||= begin
        u = URI.parse(ENV['SENTRY_DSN'])
        u.password = nil
        u.to_s
      rescue StandardError
        nil
      end
    end

    initializer 'kubelink.sentry_integration' do
      ActiveSupport.on_load :action_view do
        include Helpers
      end

      if Sentrylink.sentry_dsn.present?
        require 'sentry-ruby'
        require 'sentry-rails'

        Sentry.init do |config|
          config.breadcrumbs_logger = [:active_support_logger]
          config.dsn = Sentrylink.sentry_dsn
          config.release = ENV['RAILS_COMMIT'] if ENV.key?('RAILS_COMMIT')
          config.send_default_pii = true
          config.traces_sample_rate = 1.0
        end

        ActiveSupport.on_load :action_controller do
          include Controller
        end
      elsif Rails.env.production? && !defined?(Rake)
        throw 'This application cannot run in production without setting SENTRY_DSN'
      end
    end

    module Helpers
      def sentry_include_tag
        return unless Sentrylink.sentry_dsn.present?

        script = 'https://browser.sentry-cdn.com/5.11.0/bundle.min.js'

        [javascript_include_tag(script, crossorigin: :anonymous),
         javascript_tag(nonce: true) do
           %{Sentry.init({ dsn: "#{Sentrylink.sentry_dsn}" });}.html_safe
         end].join("\n").html_safe
      end
    end

    module Controller
      extend ActiveSupport::Concern

      included do
        before_action :set_sentry_context
      end

      def set_sentry_context
        Sentry.set_extras(params: params.dup.permit!.to_hash, url: request.url)
        Sentry.set_user(ip_address: request.remote_ip)
      end
    end
  end
end
